import React, { useCallback, useMemo } from 'react';
import { View, Image, Text, TouchableOpacity } from 'react-native';
import { useNavigation } from '@react-navigation/native';

import { getImageUri, photoStyles, smallPhotoHeight, smallPhotoWidth } from '../utils';
import { AllRegions, PhotosData } from '../utils/types';
import { NAVIGATION_PAGES } from 'src/types';
import { styles } from './styles';

import ChevronRightIcon from '../../../../../assets/icons/travels-screens/chevron-right.svg';

export const PhotoItem = React.memo(
  ({ item, allRegions }: { item: any; allRegions: AllRegions[] }) => {
    const navigation = useNavigation();

    const renderPhotos = () => {
      const photos = item.photos.slice(0, 3);
      const morePhotosCount = item.photos.length - 3;

      if (photos.length === 1) {
        return (
          <Image source={{ uri: getImageUri(photos[0].url_small) }} style={photoStyles.onePhoto} />
        );
      } else if (photos.length === 2) {
        return photos.map((photo: PhotosData, index: number) => (
          <Image
            key={photo.id}
            source={{ uri: getImageUri(photo.url_small) }}
            style={[photoStyles.twoPhotos, index === 0 && { marginRight: 8 }]}
          />
        ));
      } else {
        return (
          <View style={{ flexDirection: 'row' }}>
            <Image
              source={{ uri: getImageUri(photos[0].url_small) }}
              style={photoStyles.bigPhoto}
            />
            <View style={styles.smallPhotoContainer}>
              <Image
                source={{ uri: getImageUri(photos[1].url_small) }}
                style={photoStyles.smallPhoto}
              />
              <Image
                source={{ uri: getImageUri(photos[2].url_small) }}
                style={[photoStyles.smallPhoto, { position: 'relative' }]}
              />
              {morePhotosCount > 0 && (
                <View
                  style={[
                    styles.morePhotosOverlay,
                    { width: smallPhotoWidth, height: smallPhotoHeight }
                  ]}
                >
                  <Text style={styles.morePhotosText}>+{morePhotosCount}</Text>
                </View>
              )}
            </View>
          </View>
        );
      }
    };

    const photoViews = useMemo(() => renderPhotos(), [item.photos]);

    const handlePress = useCallback(() => {
      navigation.navigate(
        ...([
          NAVIGATION_PAGES.MORE_PHOTOS,
          {
            data: item,
            allRegions: allRegions
          }
        ] as never)
      );
    }, [navigation, item]);

    return (
      <View style={styles.listItem}>
        <View style={[styles.listItemHeader, item.date && { alignItems: 'flex-end' }]}>
          {item.date ? (
            <View style={styles.listItemTextContainer}>
              <Text style={styles.listItemTitle}>{item.date}</Text>
            </View>
          ) : (
            <>
              <Image style={styles.flagImg} source={{ uri: getImageUri(item.flag) }} />
              <View style={styles.listItemTextContainer}>
                <Text style={styles.listItemTitle}>{item.country}</Text>
                {item.name && <Text style={styles.listItemSubtitle}>{item.name}</Text>}
              </View>
            </>
          )}
          <TouchableOpacity style={styles.moreButton} onPress={handlePress}>
            <Text style={styles.moreText}>More </Text>
            <ChevronRightIcon />
          </TouchableOpacity>
        </View>
        <View style={styles.photoContainer}>{photoViews}</View>
      </View>
    );
  }
);
